from spaceone.api.repository.v1 import schema_pb2, schema_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Schema(BaseAPI, schema_pb2_grpc.SchemaServicer):

    pb2 = schema_pb2
    pb2_grpc = schema_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('SchemaService', metadata) as schema_svc:
            schema_data = schema_svc.create(params)
            return self.locator.get_info('SchemaInfo', schema_data)

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('SchemaService', metadata) as schema_svc:
            schema_data = schema_svc.update(params)
            return self.locator.get_info('SchemaInfo', schema_data)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('SchemaService', metadata) as schema_svc:
            schema_svc.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('SchemaService', metadata) as schema_svc:
            schema_data = schema_svc.get(params)
            return self.locator.get_info('SchemaInfo', schema_data)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('SchemaService', metadata) as schema_svc:
            schemas_data, total_count = schema_svc.list(params)
            return self.locator.get_info('SchemasInfo', schemas_data, total_count, minimal=self.get_minimal(params))



import logging
import re

from spaceone.core.service import *

from spaceone.repository.error import *
from spaceone.repository.manager.identity_manager import IdentityManager
from spaceone.repository.manager.policy_manager.local_policy_manager import LocalPolicyManager
from spaceone.repository.manager.repository_manager import RepositoryManager

_LOGGER = logging.getLogger(__name__)

MAX_POLICY_ID_LENGTH = 48


@authentication_handler(exclude=['get'])
@authorization_handler(exclude=['get'])
@mutation_handler
@event_handler
class PolicyService(BaseService):

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['policy_id', 'name', 'permissions', 'domain_id'])
    def create(self, params):
        """Create Policy (local repo only)

        Args:
            params (dict): {
                'policy_id': 'str',
                'name': 'str',
                'permissions': 'list',
                'labels': 'list',
                'tags': 'dict',
                'project_id': 'str', // deprecated
                'domain_id': 'str'
            }

        Returns:
            policy_info (dict)
        """

        # Pre-condition Check
        self._check_policy_id(params['policy_id'])
        self._check_project(params.get('project_id'), params['domain_id'])

        policy_mgr: LocalPolicyManager = self.locator.get_manager('LocalPolicyManager')

        # Only LOCAL repository can be created
        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        params['repository'] = repo_mgr.get_local_repository()
        params['repository_id'] = params['repository'].repository_id

        return policy_mgr.create_policy(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['policy_id', 'domain_id'])
    def update(self, params):
        """Update Policy (local repo only)

        Args:
            params (dict): {
                'policy_id': 'str',
                'name': 'str',
                'permissions': 'list',
                'labels': 'list',
                'tags': 'dict'
                'domain_id': 'str'
            }

        Returns:
            policy_info (dict)
        """

        policy_mgr: LocalPolicyManager = self.locator.get_manager('LocalPolicyManager')
        return policy_mgr.update_policy(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['policy_id', 'domain_id'])
    def delete(self, params):
        """Delete Policy (local repo only)

        Args:
            params (dict): {
                'policy_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """
        policy_id = params['policy_id']
        domain_id = params['domain_id']

        policy_mgr: LocalPolicyManager = self.locator.get_manager('LocalPolicyManager')
        return policy_mgr.delete_policy(policy_id, domain_id)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['policy_id'])
    @change_only_key({'repository_info': 'repository'})
    def get(self, params):
        """ Get Policy ((all repositories)

        Args:
            params (dict): {
                'policy_id': 'str',
                'repository_id': 'str',
                'domain_id': 'str',
                'only': 'list'
            }

        Returns:
            policy_info (dict)
        """
        policy_id = params['policy_id']
        domain_id = params.get('domain_id')
        repo_id = params.get('repository_id')
        only = params.get('only')

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        repo_vos = repo_mgr.get_all_repositories(repo_id)

        for repo_vo in repo_vos:
            _LOGGER.debug(f'[get] find at name: {repo_vo.name} '
                          f'(repo_type: {repo_vo.repository_type})')
            policy_mgr = self._get_policy_manager_by_repo(repo_vo.repository_type)

            try:
                return policy_mgr.get_policy(repo_vo, policy_id, domain_id, only)
            except Exception as e:
                _LOGGER.debug(f'[get] Can not find policy({policy_id}) at {repo_vo.name}')

        raise ERROR_NO_POLICY(policy_id=policy_id)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['repository_id'])
    @change_only_key({'repository_info': 'repository'}, key_path='query.only')
    @append_query_filter(['repository_id', 'policy_id', 'name'])
    def list(self, params):
        """ List policies (specific repository)

        Args:
            params (dict): {
                'repository_id': 'str',
                'policy_id': 'str',
                'name': 'str',
                'project_id': 'str', // deprecated
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            results (list): 'list of policy_info'
            total_count (int)
        """
        query = params.get('query', {})

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        repository_id = params['repository_id']
        repo_vo = repo_mgr.get_repository(repository_id)

        policy_mgr = self._get_policy_manager_by_repo(repo_vo.repository_type)

        return policy_mgr.list_policies(repo_vo, query, params)

    def _get_policy_manager_by_repo(self, repository_type):
        if repository_type == 'local':
            return self.locator.get_manager('LocalPolicyManager')
        elif repository_type == 'managed':
            return self.locator.get_manager('ManagedPolicyManager')
        else:
            return self.locator.get_manager('RemotePolicyManager')

    def _check_project(self, project_id, domain_id):
        if project_id:
            identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')
            identity_mgr.get_project(project_id, domain_id)

    @staticmethod
    def _check_policy_id(policy_id):
        """ Check policy id
        format of policy id: alphabet lower case, number and - (underscore is not allowed)
        """
        _LOGGER.debug(f'[_check_policy_id] policy_id => {policy_id}')
        # check policy_id
        policy_id_len = len(policy_id)
        if policy_id_len > MAX_POLICY_ID_LENGTH:
            raise ERROR_INVALID_POLICY_ID_LENGTH(length=MAX_POLICY_ID_LENGTH, policy_id=policy_id)

        # Search policy_id format
        m = re.search('(?![a-z0-9\-]).*', policy_id)
        if m:
            if len(m.group()) > 1:
                raise ERROR_INVALID_POLICY_ID_FORMAT(policy_id=policy_id)
            return True

        raise ERROR_INVALID_POLICY_ID_FORMAT(policy_id=policy_id)




from spaceone.core.error import *


class ERROR_INVALID_REPOSITORY_TYPE(ERROR_INVALID_ARGUMENT):
    _message = 'Invalid repository type. (supported_types = local | remote | managed)'


class ERROR_NOT_UPDATE_MANAGED_REPOSITORY(ERROR_INVALID_ARGUMENT):
    _message = 'Can not update managed repository.'


class ERROR_LOCAL_REPOSITORY_ALREADY_EXIST(ERROR_INVALID_ARGUMENT):
    _message = 'Local repository already exist.'


class ERROR_MANAGED_REPOSITORY_ALREADY_EXIST(ERROR_INVALID_ARGUMENT):
    _message = 'Managed repository already exist.'


class ERROR_REMOTE_REPOSITORY_ALREADY_EXIST(ERROR_INVALID_ARGUMENT):
    _message = 'Remote repository already exist. (endpoint = {endpoint})'


class ERRROR_NOT_SET_UP_REMOTE_REPOSITORY(ERROR_INVALID_ARGUMENT):
    _message = 'Remote repository is not set up.'



DATABASES = {
    'default': {
        # 'db': 'repository',
        # 'host': 'localhost',
        # 'port': 27017,
        # 'username': 'repository',
        # 'password': ''
    }
}

CACHES = {
    'default': {}
}

CONNECTORS = {
    'AWSPrivateECRConnector': {
        'aws_access_key_id': '',
        'aws_secret_access_key': '',
        'region_name': '',
        'account_id': ''
    },
    'DockerHubConnector': {},
    'HarborConnector': {
        'base_url': '',
        'token': '',
        'image_prefix': '',
        'verify': True
    },
    'RemoteRepositoryConnector': {},
    'SpaceConnector': {
        'backend': 'spaceone.core.connector.space_connector.SpaceConnector',
        'endpoints': {
            'identity': 'grpc://identity:50051',
            'secret': 'grpc://secret:50051'
        }
    }
}

REGISTRY_INFO = {
    'DOCKER_HUB': {
        'url': 'registry.hub.docker.com'
    },
    'AWS_PRIVATE_ECR': {
        'url': '',
        'image_pull_secrets': ''
    },
    'HARBOR': {
        'url': '',
        'image_pull_secrets': ''
    }
}

# Use managed repository (Read Only), if you can not use plugin marketplace
ENABLE_MANAGED_REPOSITORY = False
DEFAULT_REGISTRY = "DOCKER_HUB"  # DOCKER_HUB | AWS_PRIVATE_ECR | HARBOR

ROOT_TOKEN = ""
ROOT_TOKEN_INFO = {}

MANAGED_REGISTRY_TYPE = 'DOCKER_HUB'
MANAGED_REGISTRY_CONFIG = {}
MANAGED_PLUGIN_IMAGE_PREFIX = 'spaceone'



from mongoengine import *
from datetime import datetime

from spaceone.core.error import *
from spaceone.core.model.mongo_model import MongoModel

__all__ = ['Repository']


class Repository(MongoModel):
    repository_id = StringField(max_length=40, required=True)
    name = StringField(max_length=255)
    repository_type = StringField(max_length=255, choices=['local', 'remote', 'managed'])
    state = StringField(max_length=20, default='ENABLED')
    order = IntField(required=True)
    endpoint = StringField(max_length=255)
    version = StringField(max_length=16)
    secret_id = StringField(max_length=255)
    created_at = DateTimeField(auto_now_add=True)
    deleted_at = DateTimeField(default=None, null=True)

    meta = {
        'updatable_fields': [
            'name'
        ],
        'minimal_fields': [
            'repository_id',
            'name',
            'repository_type'
        ],
        'ordering': ['order', 'name'],
        'indexes': [
            'repository_id',
            'name',
            'repository_type'
        ]
    }


import logging
import os
import copy
import pandas as pd
import re

from spaceone.core import utils
from spaceone.repository.error import *
from spaceone.repository.manager.policy_manager import PolicyManager
from spaceone.repository.model.repository_model import Repository


__all__ = ['ManagedPolicyManager']

_LOGGER = logging.getLogger(__name__)
_BASE_DIR = os.path.join(os.path.dirname(__file__), '../../managed_resource/policy/')
_MANAGED_POLICIES = []

for filename in os.listdir(_BASE_DIR):
    if filename.endswith('.yaml'):
        file_path = os.path.join(_BASE_DIR, filename)
        policy_info = utils.load_yaml_from_file(file_path)
        _MANAGED_POLICIES.append(policy_info)


class ManagedPolicyManager(PolicyManager):

    def __init__(self):
        super().__init__()
        self.managed_policy_df = self._load_managed_policies()

    def get_policy(self, repo_vo: Repository, policy_id, domain_id, only=None):
        managed_policy_df = self._filter_managed_policies(policy_id)

        if len(managed_policy_df) == 0:
            raise ERROR_NOT_FOUND(key='policy_id', value=policy_id)

        managed_policies_info = managed_policy_df.to_dict('records')

        return self.change_response(managed_policies_info[0], repo_vo, domain_id)

    def list_policies(self, repo_vo: Repository, query: dict, params: dict):
        policy_id = params.get('policy_id')
        name = params.get('name')
        domain_id = params.get('domain_id')
        sort = query.get('sort', {})
        page = query.get('page', {})
        keyword = query.get('keyword')

        managed_policy_df = self._filter_managed_policies(policy_id, name, keyword)
        managed_policy_df = self._sort_managed_policies(managed_policy_df, sort)

        total_count = len(managed_policy_df)
        managed_policy_df = self._page_managed_policies(managed_policy_df, page)

        results = []
        for managed_policy_info in managed_policy_df.to_dict('records'):
            results.append(self.change_response(managed_policy_info, repo_vo, domain_id))

        return results, total_count

    @staticmethod
    def _load_managed_policies():
        return pd.DataFrame(_MANAGED_POLICIES)

    def _filter_managed_policies(self, policy_id=None, name=None, keyword=None):
        managed_policy_df = copy.deepcopy(self.managed_policy_df)

        if policy_id:
            managed_policy_df = managed_policy_df[managed_policy_df['policy_id'] == policy_id]

        if name:
            managed_policy_df = managed_policy_df[managed_policy_df['name'] == name]

        if keyword:
            managed_policy_df = managed_policy_df[
                managed_policy_df['name'].str.contains(keyword, flags=re.IGNORECASE)]

        return managed_policy_df

    @staticmethod
    def _sort_managed_policies(managed_policy_df: pd.DataFrame, sort: dict):
        if sort_key := sort.get('key'):
            desc = sort.get('desc', False)
            try:
                return managed_policy_df.sort_values(by=sort_key, ascending=not desc)
            except Exception as e:
                raise ERROR_SORT_KEY(sort_key=sort_key)
        else:
            return managed_policy_df

    @staticmethod
    def _page_managed_policies(managed_policy_df: pd.DataFrame, page: dict):
        if limit := page.get('limit'):
            start = page.get('start', 1) - 1
            return managed_policy_df[start:start + limit]
        else:
            return managed_policy_df



import functools
from spaceone.api.repository.v1 import plugin_pb2, repository_pb2
from spaceone.core import config
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils

__all__ = ['PluginInfo', 'PluginsInfo', 'VersionsInfo']


def PluginInfo(plugin_info: dict, minimal=False):
    info = {
        'plugin_id': plugin_info.get('plugin_id'),
        'name': plugin_info.get('name'),
        'state': plugin_info.get('state'),
        'image': plugin_info.get('image'),
        'service_type': plugin_info.get('service_type'),
        'registry_type': plugin_info.get('registry_type'),
        'provider': plugin_info.get('provider'),
    }
    if not minimal:
        registry_info = config.get_global('REGISTRY_INFO', {}).get(plugin_info.get('registry_type'))

        info.update({
            'registry_config': change_struct_type(registry_info),
            'capability': change_struct_type(plugin_info.get('capability')),
            'template': change_struct_type(plugin_info.get('template')),
            'tags': change_struct_type(plugin_info.get('tags')),
            'labels': change_list_value_type(plugin_info.get('labels')),
            'project_id': plugin_info.get('project_id'),
            'domain_id': plugin_info.get('domain_id'),
            'created_at': utils.datetime_to_iso8601(plugin_info.get('created_at')),
            'updated_at': utils.datetime_to_iso8601(plugin_info.get('updated_at'))
        })

        if 'registry_url' in plugin_info:
            info['registry_url'] = plugin_info['registry_url']
        else:
            info['registry_url'] = registry_info.get('url')

        if repository_info := plugin_info.get('repository_info'):
            info.update({
                'repository_info': repository_pb2.RepositoryInfo(
                    repository_id=repository_info.get('repository_id'),
                    name=repository_info.get('name'),
                    repository_type=repository_info.get('repository_type'),
                )
            })

    return plugin_pb2.PluginInfo(**info)


def PluginsInfo(plugins_info, total_count, **kwargs):
    return plugin_pb2.PluginsInfo(results=list(
        map(functools.partial(PluginInfo, **kwargs), plugins_info)), total_count=total_count)


def VersionsInfo(version_list):
    return plugin_pb2.VersionsInfo(results=version_list, total_count=len(version_list))



from spaceone.api.repository.v1 import policy_pb2, policy_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Policy(BaseAPI, policy_pb2_grpc.PolicyServicer):

    pb2 = policy_pb2
    pb2_grpc = policy_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('PolicyService', metadata) as policy_svc:
            policy_data = policy_svc.create(params)
            return self.locator.get_info('PolicyInfo', policy_data)

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('PolicyService', metadata) as policy_svc:
            policy_data = policy_svc.update(params)
            return self.locator.get_info('PolicyInfo', policy_data)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('PolicyService', metadata) as policy_svc:
            policy_svc.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('PolicyService', metadata) as policy_svc:
            policy_data = policy_svc.get(params)
            return self.locator.get_info('PolicyInfo', policy_data)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('PolicyService', metadata) as policy_svc:
            policies_data, total_count = policy_svc.list(params)
            return self.locator.get_info('PoliciesInfo', policies_data, total_count, minimal=self.get_minimal(params))




import logging
from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.repository.model.repository_model import Repository
from spaceone.repository.manager.plugin_manager import PluginManager

__all__ = ['RemotePluginManager']

_LOGGER = logging.getLogger(__name__)


class RemotePluginManager(PluginManager):

    def get_plugin(self, repo_vo: Repository, plugin_id, domain_id, only=None):
        endpoint = repo_vo.endpoint
        remote_repo_conn: SpaceConnector = self.locator.get_connector('SpaceConnector', endpoint=endpoint)
        plugin_info = remote_repo_conn.dispatch('Plugin.get', {
            'plugin_id': plugin_id,
            'only': only
        })

        return self.change_response(plugin_info, repo_vo, domain_id)

    def list_plugins(self, repo_vo: Repository, query: dict, params: dict):
        domain_id = params.get('domain_id')
        endpoint = repo_vo.endpoint
        remote_repo_conn: SpaceConnector = self.locator.get_connector('SpaceConnector', endpoint=endpoint)

        response = remote_repo_conn.dispatch('Plugin.list', {
            'query': query,
            'repository_id': repo_vo.repository_id
        })

        plugins_info = response.get('results', [])
        total_count = response.get('total_count', 0)
        results = []
        for plugin_info in plugins_info:
            results.append(self.change_response(plugin_info, repo_vo, domain_id))

        return results, total_count

    def get_plugin_versions(self, repo_vo: Repository, plugin_id, domain_id):
        endpoint = repo_vo.endpoint
        remote_repo_conn: SpaceConnector = self.locator.get_connector('SpaceConnector', endpoint=endpoint)
        response = remote_repo_conn.dispatch('Plugin.get_versions', {
            'plugin_id': plugin_id
        })

        versions = response.get('results', [])
        total_count = response.get('total_count', 0)

        _LOGGER.debug(f'[get_plugin_version] total version count: {total_count}')

        return versions



from spaceone.core.pygrpc.server import GRPCServer
from .repository import Repository
from .plugin import Plugin
from .schema import Schema
from .policy import Policy


_all_ = ['app']

app = GRPCServer()
app.add_service(Repository)
app.add_service(Plugin)
app.add_service(Schema)
app.add_service(Policy)



import logging

from spaceone.repository.model.policy_model import Policy
from spaceone.repository.model.repository_model import Repository
from spaceone.repository.manager.policy_manager import PolicyManager

__all__ = ['LocalPolicyManager']

_LOGGER = logging.getLogger(__name__)


class LocalPolicyManager(PolicyManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.policy_model: Policy = self.locator.get_model('Policy')

    def create_policy(self, params):
        def _rollback(policy_vo):
            policy_vo.delete()

        policy_vo = self.policy_model.create(params)
        self.transaction.add_rollback(_rollback, policy_vo)

        policy_info = policy_vo.to_dict()
        return self.change_response(policy_info, policy_vo.repository)

    def update_policy(self, params):
        policy_vo = self.policy_model.get(policy_id=params['policy_id'], domain_id=params['domain_id'])
        return self.update_policy_by_vo(params, policy_vo)

    def update_policy_by_vo(self, params, policy_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[ROLLBACK] Revert Policy Data : {old_data["name"]} ({old_data["policy_id"]})')
            policy_vo.update(old_data)

        self.transaction.add_rollback(_rollback, policy_vo.to_dict())

        policy_vo = policy_vo.update(params)
        policy_info = policy_vo.to_dict()
        return self.change_response(policy_info, policy_vo.repository)

    def delete_policy(self, policy_id, domain_id):
        policy_vo = self.policy_model.get(domain_id=domain_id, policy_id=policy_id)
        policy_vo.delete()

    def get_policy(self, repo_vo: Repository, policy_id, domain_id, only=None):
        policy_vo = self.policy_model.get(policy_id=policy_id, domain_id=domain_id, only=only)

        policy_info = policy_vo.to_dict()
        return self.change_response(policy_info, policy_vo.repository)

    def list_policies(self, repo_vo: Repository, query: dict, params: dict):
        domain_id = params.get('domain_id')
        keyword = query.get('keyword')
        query_filter = query.get('filter', [])
        query_filter_or = query.get('filter_or', [])
        query['filter'] = self._append_domain_filter(query_filter, domain_id)
        query['filter_or'] = self._append_keyword_filter(query_filter_or, keyword)

        policy_vos, total_count = self.policy_model.query(**query)
        results = []
        for policy_vo in policy_vos:
            policy_info = policy_vo.to_dict()
            results.append(self.change_response(policy_info, policy_vo.repository))

        return results, total_count

    def stat_policies(self, query):
        return self.policy_model.stat(**query)

    @staticmethod
    def _append_domain_filter(query_filter, domain_id=None):
        if domain_id:
            query_filter.append({
                'k': 'domain_id',
                'v': domain_id,
                'o': 'eq'
            })

        return query_filter

    @staticmethod
    def _append_keyword_filter(query_filter_or, keyword):
        if keyword:
            query_filter_or.append({
                'k': 'name',
                'v': keyword,
                'o': 'contain'
            })

        return query_filter_or



import logging

from spaceone.core.error import *

from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.repository.error import *
from spaceone.repository.manager.repository_manager import RepositoryManager

_LOGGER = logging.getLogger(__name__)


class RemoteRepositoryManager(RepositoryManager):

    def register_repository(self, params):
        endpoint = params.get('endpoint')

        if endpoint is None:
            raise ERROR_REQUIRED_PARAMETER(key='endpoint')

        remote_repo_vos = self.filter_repositories(repository_type="remote", endpoint=endpoint)
        if remote_repo_vos.count() > 0:
            raise ERROR_REMOTE_REPOSITORY_ALREADY_EXIST(endpoint=endpoint)

        remote_repo_conn: SpaceConnector = self.locator.get_connector('SpaceConnector', endpoint=endpoint)
        response = remote_repo_conn.dispatch('Repository.list', {'repository_type': 'local'})
        total_count = response.get('total_count', 0)

        if total_count == 0:
            raise ERRROR_NOT_SET_UP_REMOTE_REPOSITORY()

        remote_repo_info = response['results'][0]

        # Overwrite repository_id to Remote one
        params['repository_id'] = remote_repo_info['repository_id']
        params['order'] = 2

        return self.repo_model.create(params)



from spaceone.core.error import *


class ERROR_NO_SCHEMA(ERROR_BASE):
    _message = 'Schema does not exists. (name = {name})'


class ERROR_INVALID_SCHEMA(ERROR_INVALID_ARGUMENT):
    _message = 'Schema format(JSON SCHEMA) is invalid. (key = {key})'



from spaceone.repository.manager.identity_manager import *
from spaceone.repository.manager.repository_manager import *
from spaceone.repository.manager.repository_manager.managed_repository_manager import ManagedRepositoryManager
from spaceone.repository.manager.repository_manager.local_repository_manager import LocalRepositoryManager
from spaceone.repository.manager.repository_manager.remote_repository_manager import RemoteRepositoryManager
from spaceone.repository.manager.plugin_manager.managed_plugin_manager import ManagedPluginManager
from spaceone.repository.manager.plugin_manager.local_plugin_manager import LocalPluginManager
from spaceone.repository.manager.plugin_manager.remote_plugin_manager import RemotePluginManager
from spaceone.repository.manager.schema_manager.managed_schema_manager import ManagedSchemaManager
from spaceone.repository.manager.schema_manager.local_schema_manager import LocalSchemaManager
from spaceone.repository.manager.schema_manager.remote_schema_manager import RemoteSchemaManager
from spaceone.repository.manager.policy_manager.managed_policy_manager import ManagedPolicyManager
from spaceone.repository.manager.policy_manager.local_policy_manager import LocalPolicyManager
from spaceone.repository.manager.policy_manager.remote_policy_manager import RemotePolicyManager



import logging
import jsonschema

from spaceone.core.service import *

from spaceone.repository.error import *
from spaceone.repository.manager.schema_manager.local_schema_manager import LocalSchemaManager
from spaceone.repository.manager.repository_manager import RepositoryManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler(exclude=['get'])
@authorization_handler(exclude=['get'])
@mutation_handler
@event_handler
class SchemaService(BaseService):

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'service_type', 'schema', 'domain_id'])
    def create(self, params):
        """Create Schema

        Args:
            params (dict): {
                'name': 'str',
                'service_type': 'str',
                'schema': 'dict',
                'labels': 'list',
                'tags': 'dict',
                'project_id': 'str', // deprecated
                'domain_id': 'str'
            }

        Returns:
            schema_info (dict)
        """

        # Pre-condition Check
        self._check_schema(params['schema'])
        self._check_service_type(params.get('service_type'))

        schema_mgr: LocalSchemaManager = self.locator.get_manager('LocalSchemaManager')

        # Only LOCAL repository can be registered
        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        params['repository'] = repo_mgr.get_local_repository()
        params['repository_id'] = params['repository'].repository_id

        return schema_mgr.register_schema(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'domain_id'])
    def update(self, params):
        """Update Schema. (local repo only)

        Args:
            params (dict): {
                'name': 'str',
                'schema': 'dict',
                'labels': 'list',
                'tags': 'dict'
                'domain_id': 'str'
            }

        Returns:
            schema_info (dict)
        """

        # Pre-condition Check
        self._check_schema(params.get('schema'))
        self._check_service_type(params.get('service_type'))

        schema_mgr: LocalSchemaManager = self.locator.get_manager('LocalSchemaManager')
        return schema_mgr.update_schema(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'domain_id'])
    def delete(self, params):
        """Delete Schema (local repo only)

        Args:
            params (dict): {
                'name': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """
        schema_name = params['name']
        domain_id = params['domain_id']

        schema_mgr: LocalSchemaManager = self.locator.get_manager('LocalSchemaManager')
        return schema_mgr.delete_schema(schema_name, domain_id)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name'])
    @change_only_key({'repository_info': 'repository'})
    def get(self, params):
        """ Get Schema (all repositories)

        Args:
            params (dict): {
                'name': 'str',
                'repository_id': 'str',
                'domain_id': 'str',
                'only': 'list'
            }

        Returns:
            schema_vo (object)
        """

        schema_name = params['name']
        domain_id = params.get('domain_id')
        repo_id = params.get('repository_id')
        only = params.get('only')

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        repo_vos = repo_mgr.get_all_repositories(repo_id)

        for repo_vo in repo_vos:
            _LOGGER.debug(f'[get] find at name: {repo_vo.name} '
                          f'(repo_type: {repo_vo.repository_type})')
            schema_mgr = self._get_schema_manager_by_repo(repo_vo.repository_type)
            try:
                return schema_mgr.get_schema(repo_vo, schema_name, domain_id, only)
            except Exception as e:
                _LOGGER.debug(f'[get] Can not find schema({schema_name}) at {repo_vo.name}')

        raise ERROR_NO_SCHEMA(name=schema_name)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['repository_id'])
    @change_only_key({'repository_info': 'repository'}, key_path='query.only')
    @append_query_filter(['repository_id', 'name', 'service_type'])
    def list(self, params):
        """ List schemas (specific repository)

        Args:
            params (dict): {
                'repository_id': 'str',
                'name': 'str',
                'service_type': 'str',
                'project_id': 'str', // deprecated
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            results (list): 'list of schema_info'
            total_count (int)
        """

        query = params.get('query', {})

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        repository_id = params['repository_id']
        repo_vo = repo_mgr.get_repository(repository_id)

        schema_mgr = self._get_schema_manager_by_repo(repo_vo.repository_type)

        return schema_mgr.list_schemas(repo_vo, query, params)

    def _get_schema_manager_by_repo(self, repository_type):
        if repository_type == 'local':
            return self.locator.get_manager('LocalSchemaManager')
        elif repository_type == 'managed':
            return self.locator.get_manager('ManagedSchemaManager')
        else:
            return self.locator.get_manager('RemoteSchemaManager')

    @staticmethod
    def _check_schema(schema):
        """
        Check json schema
        """
        if schema:
            try:
                jsonschema.Draft7Validator.check_schema(schema)
            except Exception as e:
                raise ERROR_INVALID_SCHEMA(key='schema')

    @staticmethod
    def _check_service_type(name):
        """
        service_type has format rule
        format:
            <service>.<purpose>
        example:
            identity.Domain
            inventory.Collector

        Raises:
            ERROR_INVALID_PARAMETER
        """
        pass



import functools
from spaceone.api.repository.v1 import policy_pb2, repository_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils

__all__ = ['PolicyInfo', 'PoliciesInfo']


def PolicyInfo(policy_info, minimal=False):
    info = {
        'policy_id': policy_info.get('policy_id'),
        'name': policy_info.get('name'),
    }

    if not minimal:
        info.update({
            'state': policy_info.get('state'),
            'permissions': change_list_value_type(policy_info.get('permissions')),
            'tags': change_struct_type(policy_info.get('tags')),
            'labels': change_list_value_type(policy_info.get('labels')),
            'project_id': policy_info.get('project_id'),
            'domain_id': policy_info.get('domain_id'),
            'created_at': utils.datetime_to_iso8601(policy_info.get('created_at')),
            'updated_at': utils.datetime_to_iso8601(policy_info.get('updated_at'))
        })

        if repository_info := policy_info.get('repository_info'):
            info.update({
                'repository_info': repository_pb2.RepositoryInfo(
                    repository_id=repository_info.get('repository_id'),
                    name=repository_info.get('name'),
                    repository_type=repository_info.get('repository_type'),
                )
            })

    return policy_pb2.PolicyInfo(**info)


def PoliciesInfo(policies_info, total_count, **kwargs):
    return policy_pb2.PoliciesInfo(results=list(
        map(functools.partial(PolicyInfo, **kwargs), policies_info)), total_count=total_count)



import logging

from spaceone.core.service import *
from spaceone.repository.error import *
from spaceone.repository.manager import RepositoryManager, LocalRepositoryManager, ManagedRepositoryManager, \
    RemoteRepositoryManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class RepositoryService(BaseService):

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'repository_type'])
    def register(self, params):
        """ Register repository
        Args:
            params (dict):
                'name': 'str',
                'repository_type': 'local | remote | managed',
                'endpoint': 'str',
                'domain_id': 'str'

        Returns:
            repository_vo (object)
        """

        repo_type = params.get('repository_type')
        if repo_type == 'local':
            repo_mgr: LocalRepositoryManager = self.locator.get_manager('LocalRepositoryManager')
        elif repo_type == 'managed':
            repo_mgr: ManagedRepositoryManager = self.locator.get_manager('ManagedRepositoryManager')
        elif repo_type == 'remote':
            repo_mgr: RemoteRepositoryManager = self.locator.get_manager('RemoteRepositoryManager')
        else:
            raise ERROR_INVALID_REPOSITORY_TYPE()

        return repo_mgr.register_repository(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['repository_id'])
    def update(self, params):
        """ Update repository
        Args:
            params (dict):
                'repository_id': 'str',
                'name': 'str',
                'repository_type': 'local | remote | managed',
                'endpoint': 'str',
                'domain_id': 'str'

        Returns:
            repository_vo (object)
        """

        if params['repository_id'] == 'repo-managed':
            raise ERROR_NOT_UPDATE_MANAGED_REPOSITORY()

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        return repo_mgr.update_repository(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['repository_id'])
    def deregister(self, params):
        """ Deregister repository
        Args:
            params (dict):
                'repository_id': 'str',
                'domain_id': 'str'

        Returns:
            None
        """

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')

        return repo_mgr.delete_repository(params['repository_id'])

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['repository_id'])
    def get(self, params):
        """ Get repository
        Args:
            params (dict):
                'repository_id': 'str',
                'domain_id': 'str'

        Returns:
            repository_vo (object)
        """

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        return repo_mgr.get_repository(params['repository_id'], params.get('only'))

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @append_query_filter(['repository_id', 'name', 'repository_type'])
    @append_keyword_filter(['repository_id', 'name'])
    def list(self, params):
        """ List repositories
        Args:
            params (dict):
                'repository_id': 'str',
                'name': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'

        Returns:
            results (list): 'list of repository_vo'
            total_count (int)
        """

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        query = params.get('query', {})
        return repo_mgr.list_repositories(query)



import logging
import os
import copy
import pandas as pd
import re

from spaceone.core import utils, config
from spaceone.repository.error import *
from spaceone.repository.manager.plugin_manager import PluginManager
from spaceone.repository.model.repository_model import Repository


__all__ = ['ManagedPluginManager']

_LOGGER = logging.getLogger(__name__)
_REGISTRY_CONNECTOR_MAP = {
    'DOCKER_HUB': 'DockerHubConnector',
    'AWS_PRIVATE_ECR': 'AWSPrivateECRConnector',
    'HARBOR': 'HarborConnector'
}
_BASE_DIR = os.path.join(os.path.dirname(__file__), '../../managed_resource/plugin/')
_MANAGED_PLUGINS = []

for filename in os.listdir(_BASE_DIR):
    if filename.endswith('.yaml'):
        file_path = os.path.join(_BASE_DIR, filename)
        plugin_info = utils.load_yaml_from_file(file_path)
        _MANAGED_PLUGINS.append(plugin_info)


class ManagedPluginManager(PluginManager):

    def __init__(self):
        super().__init__()
        self.managed_registry_type = config.get_global('MANAGED_REGISTRY_TYPE', 'DOCKER_HUB')
        self.managed_registry_config = config.get_global('MANAGED_REGISTRY_CONFIG', {})
        self.managed_plugin_image_prefix = config.get_global('MANAGED_PLUGIN_IMAGE_PREFIX', 'cloudforet')
        self.managed_plugin_df = self._load_managed_plugins()

    def get_plugin(self, repo_vo: Repository, plugin_id, domain_id, only=None):
        managed_plugin_df = self._filter_managed_plugins(plugin_id)

        if len(managed_plugin_df) == 0:
            raise ERROR_NOT_FOUND(key='plugin_id', value=plugin_id)

        managed_plugins_info = managed_plugin_df.to_dict('records')

        return self.change_managed_plugin_info(managed_plugins_info[0], repo_vo, domain_id)

    def list_plugins(self, repo_vo: Repository, query: dict, params: dict):
        plugin_id = params.get('plugin_id')
        name = params.get('name')
        service_type = params.get('service_type')
        provider = params.get('provider')
        domain_id = params.get('domain_id')
        sort = query.get('sort', {})
        page = query.get('page', {})
        keyword = query.get('keyword')

        managed_plugin_df = self._filter_managed_plugins(plugin_id, name, service_type, provider, keyword)
        managed_plugin_df = self._sort_managed_plugins(managed_plugin_df, sort)

        total_count = len(managed_plugin_df)
        managed_plugin_df = self._page_managed_plugins(managed_plugin_df, page)

        results = []
        for managed_plugin_info in managed_plugin_df.to_dict('records'):
            results.append(self.change_managed_plugin_info(managed_plugin_info, repo_vo, domain_id))

        return results, total_count

    def get_plugin_versions(self, repo_vo: Repository, plugin_id, domain_id):
        managed_plugin_info = self.get_plugin(repo_vo, plugin_id, domain_id)
        registry_type = managed_plugin_info['registry_type']
        image = managed_plugin_info['image']

        registry_url = config.get_global('REGISTRY_INFO', {}).get(registry_type).get('url')
        registry_connector = _REGISTRY_CONNECTOR_MAP[registry_type]

        try:
            connector = self.locator.get_connector(registry_connector)
            tags = connector.get_tags(registry_url, image)
        except Exception as e:
            _LOGGER.error(f'[get_plugin_versions] get_tags error: {e}', exc_info=True)
            raise e
        else:
            return tags


    @staticmethod
    def _load_managed_plugins():
        managed_plugin_df = pd.DataFrame(copy.deepcopy(_MANAGED_PLUGINS))
        return managed_plugin_df.fillna('')

    def _filter_managed_plugins(self, plugin_id=None, name=None, service_type=None, provider=None, keyword=None):
        managed_plugin_df = copy.deepcopy(self.managed_plugin_df)

        if plugin_id:
            managed_plugin_df = managed_plugin_df[managed_plugin_df['plugin_id'] == plugin_id]

        if name:
            managed_plugin_df = managed_plugin_df[managed_plugin_df['name'] == name]

        if service_type:
            managed_plugin_df = managed_plugin_df[managed_plugin_df['service_type'] == service_type]

        if provider:
            managed_plugin_df = managed_plugin_df[managed_plugin_df['provider'] == provider]

        if keyword:
            managed_plugin_df = managed_plugin_df[
                managed_plugin_df['name'].str.contains(keyword, flags=re.IGNORECASE)]

        return managed_plugin_df

    @staticmethod
    def _sort_managed_plugins(managed_plugin_df: pd.DataFrame, sort: dict):
        if sort_key := sort.get('key'):
            desc = sort.get('desc', False)
            try:
                return managed_plugin_df.sort_values(by=sort_key, ascending=not desc)
            except Exception as e:
                raise ERROR_SORT_KEY(sort_key=sort_key)
        else:
            return managed_plugin_df

    @staticmethod
    def _page_managed_plugins(managed_plugin_df: pd.DataFrame, page: dict):
        if limit := page.get('limit'):
            start = page.get('start', 1) - 1
            return managed_plugin_df[start:start + limit]
        else:
            return managed_plugin_df

    def change_managed_plugin_info(self, plugin_info, repo_vo, domain_id):
        plugin_info['state'] = 'ENABLED'
        plugin_info['registry_type'] = self.managed_registry_type
        plugin_info['registry_config'] = self.managed_registry_config
        plugin_info['image'] = f"{self.managed_plugin_image_prefix}/{plugin_info['image']}"

        if repo_vo:
            plugin_info['repository_info'] = {
                'repository_id': repo_vo.repository_id,
                'name': repo_vo.name,
                'repository_type': repo_vo.repository_type,
            }

        if domain_id:
            plugin_info['domain_id'] = domain_id

        return plugin_info



from spaceone.repository.error import *
from spaceone.repository.manager.repository_manager import RepositoryManager


class ManagedRepositoryManager(RepositoryManager):

    def register_repository(self, params):
        # Assume there is only one local repository
        managed_repo_vo = self.get_managed_repository()
        if managed_repo_vo is not None:
            raise ERROR_MANAGED_REPOSITORY_ALREADY_EXIST()

        params['repository_id'] = 'repo-managed'
        params['order'] = 1
        return self.repo_model.create(params)



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel
from spaceone.repository.model.repository_model import Repository

__all__ = ['Plugin']


class Plugin(MongoModel):
    """
    name is unique per domain
    """
    plugin_id = StringField(max_length=255, unique_with='domain_id')
    name = StringField(max_length=255)
    state = StringField(max_length=40, default='ENABLED', choices=('ENABLED', 'DISABLED'))
    image = StringField(max_length=255)
    registry_type = StringField(max_length=255, default='DOCKER_HUB')
    registry_config = DictField()
    service_type = StringField(max_length=255)
    provider = StringField(max_length=255, default=None, null=True)
    capability = DictField()
    template = DictField()
    labels = ListField(StringField(max_length=255))
    tags = DictField()
    repository = ReferenceField('Repository', reverse_delete_rule=DENY)
    repository_id = StringField(max_length=40)
    project_id = StringField(max_length=255, default=None, null=True)
    domain_id = StringField(max_length=255)
    created_at = DateTimeField(auto_now_add=True)
    updated_at = DateTimeField(auto_now=True)
    meta = {
        'updatable_fields': [
            'name',
            'state',
            'capability',
            'template',
            'labels',
            'tags'
        ],
        'minimal_fields': [
            'plugin_id',
            'name',
            'state',
            'image',
            'registry_type',
            'service_type',
            'provider'
        ],
        'change_query_keys': {
            'repository_id': 'repository.repository_id'
        },
        'reference_query_keys': {
            'repository': Repository
        },
        'ordering': ['name'],
        'indexes': [
            'plugin_id',
            'state',
            'registry_type',
            'service_type',
            'provider',
            'repository',
            'repository_id',
            'project_id',
            'domain_id'
        ]
    }



import copy
import logging
import jsonschema
import re

from spaceone.core.service import *
from spaceone.core import config

from spaceone.repository.error import *
from spaceone.repository.model.capability_model import Capability
from spaceone.repository.manager.plugin_manager.local_plugin_manager import LocalPluginManager
from spaceone.repository.manager.repository_manager import RepositoryManager

_LOGGER = logging.getLogger(__name__)

MAX_IMAGE_NAME_LENGTH = 48
REGISTRY_MAP = {
    "DOCKER_HUB": "DockerHubConnector",
    "AWS_PRIVATE_ECR": "AWSPrivateECRConnector",
    "HARBOR": "HarborConnector"
}


@authentication_handler(exclude=['get', 'get_versions'])
@authorization_handler(exclude=['get', 'get_versions'])
@mutation_handler
@event_handler
class PluginService(BaseService):

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'service_type', 'image', 'domain_id'])
    def register(self, params):
        """Register Plugin (local repo only)

        Args:
            params (dict): {
                'name': 'str',
                'service_type': 'str',
                'registry_type': 'str',
                'registry_config': 'dict',
                'image': 'str',
                'provider': 'str',
                'capability': 'dict',
                'template': 'dict',
                'labels': 'list',
                'tags': 'dict',
                'project_id': 'str', // deprecated
                'domain_id': 'str'
            }

        Returns:
            plugin_info (dict)
        """

        registry_type = params.get('registry_type', 'DOCKER_HUB')
        image = params['image']
        plugin_id = self._get_image_name(image)

        # Pre-condition Check
        self._check_template(params.get('template'))
        # self._check_capability(params.get('capability'))
        self._check_service_type(params.get('service_type'))
        self._check_image(image)
        self._check_registry_type(registry_type)

        plugin_mgr: LocalPluginManager = self.locator.get_manager('LocalPluginManager')

        # Only LOCAL repository can be registered
        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        params['plugin_id'] = plugin_id
        params['repository'] = repo_mgr.get_local_repository()
        params['repository_id'] = params['repository'].repository_id

        return plugin_mgr.register_plugin(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['plugin_id', 'domain_id'])
    def update(self, params):
        """Update Plugin. (local repo only)

        Args:
            params (dict): {
                'plugin_id': 'str',
                'name': 'str',
                'capability': 'dict',
                'template': 'dict',
                'labels': 'list',
                'tags': 'dict'
                'domain_id': 'str'
            }

        Returns:
            plugin_info (dict)
        """

        # Pre-condition Check
        self._check_template(params.get('template'))
        # self._check_capability(params.get('capability'))
        self._check_service_type(params.get('service_type'))

        plugin_mgr: LocalPluginManager = self.locator.get_manager('LocalPluginManager')
        return plugin_mgr.update_plugin(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['plugin_id', 'domain_id'])
    def deregister(self, params):
        """Deregister Plugin (local repo only)

        Args:
            params (dict): {
                'plugin_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """

        #########################################################################
        # Warning 
        #
        # To deregister plugin, plugin has to verify there is no installed plugins
        # If there was installed plugin, other micro service can query endpoint
        # But plugin can not reply endpoint
        ##########################################################################
        # TODO: check plugin is not used

        plugin_mgr: LocalPluginManager = self.locator.get_manager('LocalPluginManager')
        return plugin_mgr.delete_plugin(params['plugin_id'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['plugin_id', 'domain_id'])
    def enable(self, params):
        """ Enable plugin (local repo only)

        Args:
            params (dict): {
                'plugin_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            plugin_info (dict)
        """

        plugin_mgr: LocalPluginManager = self.locator.get_manager('LocalPluginManager')
        return plugin_mgr.enable_plugin(params['plugin_id'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['plugin_id', 'domain_id'])
    def disable(self, params):
        """ Disable Plugin (local repo only)

        Args:
            params (dict): {
                'plugin_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            plugin_info (dict)
        """
        plugin_mgr: LocalPluginManager = self.locator.get_manager('LocalPluginManager')
        return plugin_mgr.disable_plugin(params['plugin_id'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['plugin_id'])
    def get_versions(self, params):
        """ Get Plugin version (all repositories)

        Args:
            params (dict): {
                'plugin_id': 'str',
                'repository_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            version_list (list)
        """

        plugin_id = params['plugin_id']
        domain_id = params.get('domain_id')
        repo_id = params.get('repository_id')

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        repo_vos = repo_mgr.get_all_repositories(repo_id)

        for repo_vo in repo_vos:
            _LOGGER.debug(f'[get] find at name: {repo_vo.name} '
                          f'(repo_type: {repo_vo.repository_type})')
            plugin_mgr = self._get_plugin_manager_by_repo(repo_vo.repository_type)
            try:
                return plugin_mgr.get_plugin_versions(repo_vo, plugin_id, domain_id)
            except Exception as e:
                _LOGGER.debug(f'[get_versions] Cannot find plugin: {plugin_id} at {repo_vo.name}')

        raise ERROR_NO_PLUGIN(plugin_id=plugin_id)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['plugin_id'])
    @change_only_key({'repository_info': 'repository'})
    def get(self, params):
        """ Get Plugin (all repositories)

        Args:
            params (dict): {
                'plugin_id': 'str',
                'repository_id': 'str',
                'domain_id': 'str',
                'only': 'list'
            }

        Returns:
            plugin_info (dict)
        """

        plugin_id = params['plugin_id']
        domain_id = params.get('domain_id')
        repo_id = params.get('repository_id')
        only = params.get('only')

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        repo_vos = repo_mgr.get_all_repositories(repo_id)

        for repo_vo in repo_vos:
            _LOGGER.debug(f'[get] find at name: {repo_vo.name} '
                          f'(repo_type: {repo_vo.repository_type})')
            plugin_mgr = self._get_plugin_manager_by_repo(repo_vo.repository_type)
            try:
                return plugin_mgr.get_plugin(repo_vo, plugin_id, domain_id, only)
            except Exception as e:
                _LOGGER.debug(f'[get] Can not find plugin({plugin_id}) at {repo_vo.name}')

        raise ERROR_NO_PLUGIN(plugin_id=plugin_id)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @change_only_key({'repository_info': 'repository'}, key_path='query.only')
    @append_query_filter(['repository_id', 'plugin_id', 'name', 'state', 'service_type',
                          'registry_type', 'provider'])
    def list(self, params):
        """ List plugins (all repositories)

        Args:
            params (dict): {
                'repository_id': 'str',
                'plugin_id': 'str',
                'name': 'str',
                'state': 'str',
                'service_type': 'str',
                'registry_type': 'str',
                'provider': 'str',
                'project_id': 'str', // deprecated
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            results (list): 'list of plugin_info'
            total_count (int)
        """

        query = params.get('query', {})
        query['only'] = query.get('only', [])

        if 'registry_url' in query['only']:
            query['only'].remove('registry_url')

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        if repository_id := params.get('repository_id'):

            repo_vo = repo_mgr.get_repository(repository_id)

            plugin_mgr = self._get_plugin_manager_by_repo(repo_vo.repository_type)

            return plugin_mgr.list_plugins(repo_vo, query, params)
        else:
            all_plugins_info = []
            plugin_total_count = 0
            query_page = None
            if 'page' in query:
                query_page = query['page']
                del query['page']

            repo_vos = repo_mgr.get_all_repositories()
            for repo_vo in repo_vos:
                plugin_mgr = self._get_plugin_manager_by_repo(repo_vo.repository_type)
                plugins_info, total_count = plugin_mgr.list_plugins(repo_vo, copy.deepcopy(query), params)
                all_plugins_info += plugins_info
                plugin_total_count += total_count

            if query_page:
                page_start = query_page.get('start', 1) - 1
                page_limit = query_page.get('limit')

                if page_limit:
                    all_plugins_info = all_plugins_info[page_start:page_start + page_limit]

            return all_plugins_info, plugin_total_count

    def _get_plugin_manager_by_repo(self, repository_type):
        if repository_type == 'local':
            return self.locator.get_manager('LocalPluginManager')
        elif repository_type == 'managed':
            return self.locator.get_manager('ManagedPluginManager')
        else:
            return self.locator.get_manager('RemotePluginManager')

    @staticmethod
    def _check_template(template):
        """
        Check json schema
        """
        if template:
            for key, value in template.items():
                if isinstance(value, dict) and 'schema' in value:
                    try:
                        jsonschema.Draft7Validator.check_schema(value['schema'])
                    except Exception as e:
                        raise ERROR_INVALID_TEMPLATE_SCHEMA(key=f'template.{key}')

    @staticmethod
    def _check_capability(capability):
        if capability:
            try:
                capability_vo = Capability(capability)
                capability_vo.validate()
            except Exception as e:
                raise ERROR_INVALID_PARAMETER(key='capability', reason=e)

    @staticmethod
    def _check_service_type(name):
        """
        service_type has format rule
        format:
            <service>.<purpose>
        example:
            identity.Domain
            inventory.Collector

        Raises:
            ERROR_INVALID_PARAMETER
        """
        if name:
            pass
            # idx = name.split('.')
            # if len(idx) != 2:
            #     raise ERROR_INVALID_PARAMETER(key='service_type', reason=f'{name} format is invalid.')

    @staticmethod
    def _get_image_name(image):
        return image.rsplit('/', 1)[-1]

    @staticmethod
    def _check_registry_type(registry_type):
        print(registry_type)
        registry_url = config.get_global('REGISTRY_INFO', {}).get(registry_type).get('url', '')

        if registry_url.strip() == '':
            raise ERROR_REGISTRY_SETTINGS(registry_type=registry_type)

    def _check_image(self, image):
        """ Check image name
        format: repository/image_name
        length of image_name: < 40
        format of image_name: string and - (underscore is not allowed)
        """
        _LOGGER.debug(f'[_check_image] {image}')
        image_name = self._get_image_name(image)

        # check image_name length
        if len(image_name) > MAX_IMAGE_NAME_LENGTH:
            raise ERROR_INVALID_IMAGE_LENGTH(name=image_name, length=MAX_IMAGE_NAME_LENGTH)

        # Search naming format
        m = re.search('(?![a-zA-Z0-9\-]).*', image_name)
        if m:
            if len(m.group()) > 1:
                raise ERROR_INVALID_IMAGE_NAME_FORMAT(name=image_name)
            return True

        raise ERROR_INVALID_IMAGE_NAME_FORMAT(name=image_name)



import logging
from abc import abstractmethod

from spaceone.core.manager import BaseManager
from spaceone.repository.model.repository_model import Repository

__all__ = ['SchemaManager']

_LOGGER = logging.getLogger(__name__)


class SchemaManager(BaseManager):

    def create_schema(self, params):
        pass

    def update_schema(self, params):
        pass

    def delete_schema(self, schema_id, domain_id):
        pass

    @abstractmethod
    def get_schema(self, repo_vo: Repository, schema_name, domain_id, only=None):
        pass

    @abstractmethod
    def list_schemas(self, repo_vo: Repository, query: dict, params: dict):
        pass

    def stat_schemas(self, query):
        pass

    @staticmethod
    def change_response(info, repo_vo: Repository = None, domain_id: str = None):
        if repo_vo:
            info['repository_info'] = {
                'repository_id': repo_vo.repository_id,
                'name': repo_vo.name,
                'repository_type': repo_vo.repository_type,
            }

        if domain_id:
            info['domain_id'] = domain_id

        return info



from spaceone.repository.model.plugin_model import *
from spaceone.repository.model.schema_model import *
from spaceone.repository.model.policy_model import *
from spaceone.repository.model.repository_model import *



from spaceone.core.error import *


class ERROR_NO_POLICY(ERROR_INVALID_ARGUMENT):
    _message = 'Policy does not exists. (policy_id = {policy_id})'


class ERROR_INVALID_POLICY_ID_FORMAT(ERROR_INVALID_ARGUMENT):
    _message = 'Wrong policy id. Only lower-case/number/hyphen character can be used. (policy_id = {policy_id})'


class ERROR_INVALID_POLICY_ID_LENGTH(ERROR_INVALID_ARGUMENT):
    _message = 'The length of policy_id should be less than equal {length} in {policy_id}'

class ERROR_SORT_KEY(ERROR_INVALID_ARGUMENT):
    _message = 'Sort key is invalid. (sort_key = {sort_key})'
